/**
 *
 * @param { string } el
 * @param { string } [tpl=]
 * @param { {[propName: string]: any }} [attrs={}]
 * @param { string } [cname='']
 * @returns { HTMLElement | null }
 */
export function createDom(el?: string, tpl?: string, attrs?: {
    [propName: string]: any;
}, cname?: string): HTMLElement | null;
/**
 *
 * @param { string } html
 * @param { { [propName: string]: any } } [attrs={}]
 * @param { string } [classname=""]
 * @returns { HTMLElement | null }
 */
export function createDomFromHtml(html: string, attrs?: {
    [propName: string]: any;
}, classname?: string): HTMLElement | null;
/**
 *
 * @param { HTMLElement } el
 * @param { string } className
 * @returns { boolean }
 */
export function hasClass(el: HTMLElement, className: string): boolean;
/**
 *
 * @param { HTMLElement } el
 * @param { string } className
 * @returns { void }
 */
export function addClass(el: HTMLElement, className: string): void;
/**
 *
 * @param { HTMLElement } el
 * @param { string } className
 * @returns { void }
 */
export function removeClass(el: HTMLElement, className: string): void;
/**
 *
 * @param { HTMLElement } el
 * @param { string } className
 * @returns { void }
 */
export function toggleClass(el: HTMLElement, className: string): void;
/**
 *
 * @param { string | Object } args0
 * @param { string } [className]
 * @returns { string }
 */
export function classNames(...args: any[]): string;
/**
 *
 * @param { HTMLElement } el
 * @param { string } sel
 * @returns { HTMLElement }
 */
export function findDom(el: HTMLElement, sel: string): HTMLElement;
/**
 *
 * @param { HTMLElement } dom
 * @param { string } key
 * @returns { any }
 */
export function getCss(dom: HTMLElement, key: string): any;
export function padStart(str: any, length: any, pad: any): string;
/**
 *
 * @param { number } time
 * @returns { string }
 */
export function format(time: number): string;
/**
 *
 * @param { Object } e
 * @returns { Object }
 */
export function event(e: any): any;
/**
 *
 * @param { any } obj
 * @returns { string }
 */
export function typeOf(obj: any): string;
/**
 *
 * @param { any } dst
 * @param { any } src
 * @returns { any }
 */
export function deepCopy(dst: any, src: any): any;
/**
 *
 * @param { any } dst
 * @param { any } src
 * @returns { any }
 */
export function deepMerge(dst: any, src: any): any;
export function getBgImage(el: any): string;
/**
 *
 * @param {  HTMLElement } dom
 * @returns { HTMLElement | null }
 */
export function copyDom(dom: HTMLElement): HTMLElement | null;
/**
 *
 * @param { any } context
 * @param { string } eventName
 * @param { function } intervalFunc
 * @param { number } frequency
 */
export function setInterval(context: any, eventName: string, intervalFunc: Function, frequency: number): void;
/**
 *
 * @param { any } context
 * @param { string } eventName
 * @returns { void }
 */
export function clearInterval(context: any, eventName: string): void;
/**
 *
 * @param { any } context
 * @param { function } fun
 * @param { number } time
 * @returns { number }
 */
export function setTimeout(context: any, fun: Function, time: number): number;
/**
 *
 * @param { any } context
 * @param { number } id
 */
export function clearTimeout(context: any, id: number): void;
/**
 *
 * @param { any } context
 */
export function clearAllTimers(context: any): void;
/**
 *
 * @param { string } name
 * @param { string } imgUrl
 * @param { number } [width]
 * @param { number } [height]
 * @returns { HTMLElement }
 */
export function createImgBtn(name: string, imgUrl: string, width?: number, height?: number): HTMLElement;
/**
 *
 * @param { string } hex
 * @param { string | number } alpha
 * @returns { string }
 */
export function Hex2RGBA(hex: string, alpha: string | number): string;
/**
 *
 * @returns { HTMLElement | null }
 */
export function getFullScreenEl(): HTMLElement | null;
/**
 * @param { any }
 * @returns { boolean }
 */
export function checkIsFunction(fun: any): boolean;
/**
 * @param { any }
 * @returns { boolean }
 */
export function checkIsObject(obj: any): boolean;
/**
 * @param { HTMLElement }
 */
export function hide(dom: any): void;
/**
 * @param { HTMLElement }
 * @param { block | flex | inline-block | inline-flex } [display]
 */
export function show(dom: any, display: any): void;
/**
 *
 * @param { any } val
 * @returns { boolean }
 */
export function isUndefined(val: any): boolean;
export function isNotNull(val: any): boolean;
/**
 *
 * @param { HTMLElement } dom
 * @param { string } [text]
 * @returns
 */
export function setStyleFromCsstext(dom: HTMLElement, text?: string): void;
/**
 *
 * @param { HTMLElement } dom
 * @param { Array<string> } [list] attribute names to filter
 * @returns { {} | {[propName: string]: any;} }
 */
export function filterStyleFromText(dom: HTMLElement, list?: Array<string>): {} | {
    [propName: string]: any;
};
/**
 *
 * @param { HTMLElement } dom
 * @returns { {} | {[propName: string]: any;} }
 */
export function getStyleFromCsstext(dom: HTMLElement): {} | {
    [propName: string]: any;
};
export function stopPropagation(e: any): void;
export function scrollTop(): number;
export function scrollLeft(): number;
export function checkTouchSupport(): boolean;
/**
 * @description css中有zoom的时候，位移会等比缩放，但是元素的宽高不会等比缩放，所以通过该api做统一
 * https://bugs.chromium.org/p/chromium/issues/detail?id=429140#c8
 * @param {Events} e
 * @param {number} zoom
 * @returns
 */
export function getEventPos(e: Events, zoom?: number): {
    x: number;
    y: number;
    clientX: number;
    clientY: number;
    offsetX: number;
    offsetY: number;
    pageX: number;
    pageY: number;
};
export function requestAnimationFrame(callback: any): any;
export function getHostFromUrl(url: any): string;
export function cancelAnimationFrame(frameId: any): void;
/**
 * @desc Check whether it is MediaSource start
 * @param { HTMLVideoElement | HTMLAudioElement | HTMLElement } video
 * @returns { Boolean }
 */
export function isMSE(video: HTMLVideoElement | HTMLAudioElement | HTMLElement): boolean;
export function isBlob(url: any): boolean;
/**
 * @param { number } did
 * @returns { string }
 */
export function generateSessionId(did?: number): string;
export function createEvent(eventName: any): Event;
/**
 * @description Adjust currentTime according to duration, when ended is true and currentTime is less than duration, or
 *              currentTime is greater than duration, return duration
 * @param { number } time
 * @param { number } duration
 * @param { boolean } isEnded
 * @returns { number } Adjusted time
 */
export function adjustTimeByDuration(time: number, duration: number, isEnded: boolean): number;
export function createPositionBar(className: any, root: any): HTMLElement;
export function getTransformStyle(pos?: {
    x: number;
    y: number;
    scale: number;
    rotate: number;
}): string;
/**
 * @description 角度换算
 * @param {number} val
 * @returns {number}
 */
export function convertDeg(val: number): number;
export function getIndexByTime(time: any, segments: any): number;
export function getOffsetCurrentTime(currentTime: any, segments: any, index?: number): any;
/**
 *
 * @param {*} offsetTime
 * @param {*} segments
 * @returns
 */
export function getCurrentTimeByOffset(offsetTime: any, segments: any): any;
/**
 * @returns { string }
 */
export function getLang(): string;
export function checkIsCurrentVideo(element: any, playerId: any, key: any): boolean;
