import { typeof as _typeof, toConsumableArray as _toConsumableArray } from "../_virtual/_rollupPluginBabelHelpers.js";
import XG_DEBUG from "./debug.js";
function createDom() {
  var el = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : "div";
  var tpl = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : "";
  var attrs = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : {};
  var cname = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : "";
  var dom = document.createElement(el);
  dom.className = cname;
  dom.innerHTML = tpl;
  Object.keys(attrs).forEach(function(item) {
    var key = item;
    var value = attrs[item];
    if (el === "video" || el === "audio" || el === "live-video") {
      if (value) {
        dom.setAttribute(key, value);
      }
    } else {
      dom.setAttribute(key, value);
    }
  });
  return dom;
}
function createDomFromHtml(html) {
  var attrs = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {};
  var classname = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : "";
  try {
    var doc = document.createElement("div");
    doc.innerHTML = html;
    var dom = doc.children;
    doc = null;
    if (dom.length > 0) {
      dom = dom[0];
      classname && addClass(dom, classname);
      if (attrs) {
        Object.keys(attrs).forEach(function(key) {
          dom.setAttribute(key, attrs[key]);
        });
      }
      return dom;
    }
    return null;
  } catch (err) {
    XG_DEBUG.logError("createDomFromHtml", err);
    return null;
  }
}
function hasClass(el, className) {
  if (!el || !className) {
    return false;
  }
  try {
    return Array.prototype.some.call(el.classList, function(item) {
      return item === className;
    });
  } catch (e) {
    var orgClassName = el.className && _typeof(el.className) === "object" ? el.getAttribute("class") : el.className;
    return orgClassName && !!orgClassName.match(new RegExp("(\\s|^)" + className + "(\\s|$)"));
  }
}
function addClass(el, className) {
  if (!el || !className) {
    return;
  }
  try {
    className.replace(/(^\s+|\s+$)/g, "").split(/\s+/g).forEach(function(item) {
      item && el.classList.add(item);
    });
  } catch (e) {
    if (!hasClass(el, className)) {
      if (el.className && _typeof(el.className) === "object") {
        el.setAttribute("class", el.getAttribute("class") + " " + className);
      } else {
        el.className += " " + className;
      }
    }
  }
}
function removeClass(el, className) {
  if (!el || !className) {
    return;
  }
  try {
    className.replace(/(^\s+|\s+$)/g, "").split(/\s+/g).forEach(function(item) {
      item && el.classList.remove(item);
    });
  } catch (e) {
    if (hasClass(el, className)) {
      className.split(/\s+/g).forEach(function(item) {
        var reg = new RegExp("(\\s|^)" + item + "(\\s|$)");
        if (el.className && _typeof(el.className) === "object") {
          el.setAttribute("class", el.getAttribute("class").replace(reg, " "));
        } else {
          el.className = el.className.replace(reg, " ");
        }
      });
    }
  }
}
function toggleClass(el, className) {
  if (!el) {
    return;
  }
  className.split(/\s+/g).forEach(function(item) {
    if (hasClass(el, item)) {
      removeClass(el, item);
    } else {
      addClass(el, item);
    }
  });
}
function classNames() {
  var _arguments = arguments;
  var classname = [];
  var _loop = function _loop2(i2) {
    if (typeOf(_arguments[i2]) === "String") {
      classname.push(_arguments[i2]);
    } else if (typeOf(_arguments[i2]) === "Object") {
      Object.keys(_arguments[i2]).map(function(key) {
        if (_arguments[i2][key]) {
          classname.push(key);
        }
      });
    }
  };
  for (var i = 0; i < arguments.length; i++) {
    _loop(i);
  }
  return classname.join(" ");
}
function findDom() {
  var el = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : document;
  var sel = arguments.length > 1 ? arguments[1] : void 0;
  var dom;
  try {
    dom = el.querySelector(sel);
  } catch (e) {
    XG_DEBUG.logError("export function findDom", e);
    if (sel.indexOf("#") === 0) {
      dom = el.getElementById(sel.slice(1));
    }
  }
  return dom;
}
function getCss(dom, key) {
  return dom.currentStyle ? dom.currentStyle[key] : document.defaultView.getComputedStyle(dom, false)[key];
}
function padStart(str, length, pad) {
  var charstr = String(pad);
  var len = length >> 0;
  var maxlen = Math.ceil(len / charstr.length);
  var chars = [];
  var r = String(str);
  while (maxlen--) {
    chars.push(charstr);
  }
  return chars.join("").substring(0, len - r.length) + r;
}
function format(time) {
  if (window.isNaN(time)) {
    return "";
  }
  time = Math.round(time);
  var hour = padStart(Math.floor(time / 3600), 2, 0);
  var minute = padStart(Math.floor((time - hour * 3600) / 60), 2, 0);
  var second = padStart(Math.floor(time - hour * 3600 - minute * 60), 2, 0);
  return (hour === "00" ? [minute, second] : [hour, minute, second]).join(":");
}
function event(e) {
  if (e.touches) {
    var touch = e.touches[0] || e.changedTouches[0];
    e.clientX = touch.clientX || 0;
    e.clientY = touch.clientY || 0;
    e.offsetX = touch.pageX - touch.target.offsetLeft;
    e.offsetY = touch.pageY - touch.target.offsetTop;
  }
  e._target = e.target || e.srcElement;
}
function typeOf(obj) {
  return Object.prototype.toString.call(obj).match(/([^\s.*]+)(?=]$)/g)[0];
}
function deepCopy(dst, src) {
  if (typeOf(src) === "Object" && typeOf(dst) === "Object") {
    Object.keys(src).forEach(function(key) {
      if (typeOf(src[key]) === "Object" && !(src[key] instanceof Node)) {
        if (dst[key] === void 0 || dst[key] === void 0) {
          dst[key] = src[key];
        } else {
          deepCopy(dst[key], src[key]);
        }
      } else if (typeOf(src[key]) === "Array") {
        dst[key] = typeOf(dst[key]) === "Array" ? dst[key].concat(src[key]) : src[key];
      } else {
        dst[key] = src[key];
      }
    });
    return dst;
  }
}
function deepMerge(dst, src) {
  Object.keys(src).map(function(key) {
    if (typeOf(src[key]) === "Array" && typeOf(dst[key]) === "Array") {
      if (typeOf(dst[key]) === "Array") {
        var _dst$key;
        (_dst$key = dst[key]).push.apply(_dst$key, _toConsumableArray(src[key]));
      }
    } else if (typeOf(dst[key]) === typeOf(src[key]) && dst[key] !== null && typeOf(dst[key]) === "Object" && !(src[key] instanceof window.Node)) {
      deepMerge(dst[key], src[key]);
    } else {
      src[key] !== null && (dst[key] = src[key]);
    }
  });
  return dst;
}
function getBgImage(el) {
  var url = (el.currentStyle || window.getComputedStyle(el, null)).backgroundImage;
  if (!url || url === "none") {
    return "";
  }
  var a = document.createElement("a");
  a.href = url.replace(/url\("|"\)/g, "");
  return a.href;
}
function copyDom(dom) {
  if (dom && dom.nodeType === 1) {
    var back = document.createElement(dom.tagName);
    Array.prototype.forEach.call(dom.attributes, function(node) {
      back.setAttribute(node.name, node.value);
    });
    if (dom.innerHTML) {
      back.innerHTML = dom.innerHTML;
    }
    return back;
  } else {
    return "";
  }
}
function setInterval(context, eventName, intervalFunc, frequency) {
  if (!context._interval[eventName]) {
    context._interval[eventName] = window.setInterval(intervalFunc.bind(context), frequency);
  }
}
function clearInterval(context, eventName) {
  clearInterval(context._interval[eventName]);
  context._interval[eventName] = null;
}
function setTimeout(context, fun, time) {
  if (!context._timers) {
    context._timers = [];
  }
  var id = window.setTimeout(function() {
    fun();
    clearTimeout(context, id);
  }, time);
  context._timers.push(id);
  return id;
}
function clearTimeout(context, id) {
  var _timers = context._timers;
  if (typeOf(_timers) === "Array") {
    for (var i = 0; i < _timers.length; i++) {
      if (_timers[i] === id) {
        _timers.splice(i, 1);
        window.clearTimeout(id);
        break;
      }
    }
  } else {
    window.clearTimeout(id);
  }
}
function clearAllTimers(context) {
  var _timers = context._timers;
  if (typeOf(_timers) === "Array") {
    _timers.map(function(item) {
      clearTimeout(item);
    });
    context._timerIds = [];
  }
}
function createImgBtn(name, imgUrl, width, height) {
  var btn = createDom("xg-".concat(name), "", {}, "xgplayer-".concat(name, "-img"));
  btn.style.backgroundImage = 'url("'.concat(imgUrl, '")');
  if (width && height) {
    var w, h, unit;
    ["px", "rem", "em", "pt", "dp", "vw", "vh", "vm", "%"].every(function(item) {
      if (width.indexOf(item) > -1 && height.indexOf(item) > -1) {
        w = parseFloat(width.slice(0, width.indexOf(item)).trim());
        h = parseFloat(height.slice(0, height.indexOf(item)).trim());
        unit = item;
        return false;
      } else {
        return true;
      }
    });
    btn.style.width = "".concat(w).concat(unit);
    btn.style.height = "".concat(h).concat(unit);
    btn.style.backgroundSize = "".concat(w).concat(unit, " ").concat(h).concat(unit);
    if (name === "start") {
      btn.style.margin = "-".concat(h / 2).concat(unit, " auto auto -").concat(w / 2).concat(unit);
    } else {
      btn.style.margin = "auto 5px auto 5px";
    }
  }
  return btn;
}
function Hex2RGBA(hex, alpha) {
  var rgb = [];
  if (/^\#[0-9A-F]{3}$/i.test(hex)) {
    var sixHex = "#";
    hex.replace(/[0-9A-F]/ig, function(kw) {
      sixHex += kw + kw;
    });
    hex = sixHex;
  }
  if (/^#[0-9A-F]{6}$/i.test(hex)) {
    hex.replace(/[0-9A-F]{2}/ig, function(kw) {
      rgb.push(parseInt(kw, 16));
    });
    return "rgba(".concat(rgb.join(","), ", ").concat(alpha, ")");
  } else {
    return "rgba(255, 255, 255, 0.1)";
  }
}
function getFullScreenEl() {
  return document.fullscreenElement || document.webkitFullscreenElement || document.mozFullScreenElement || document.msFullscreenElement;
}
function checkIsFunction(fun) {
  return fun && typeof fun === "function";
}
function checkIsObject(obj) {
  return obj !== null && _typeof(obj) === "object";
}
function hide(dom) {
  dom.style.display = "none";
}
function show(dom, display) {
  dom.style.display = display || "block";
}
function isUndefined(val) {
  if (typeof val === "undefined" || val === null) {
    return true;
  }
}
function isNotNull(val) {
  return !(val === void 0 || val === null);
}
function setStyleFromCsstext(dom, text) {
  if (!text) {
    return;
  }
  if (typeOf(text) === "String") {
    var styleArr = text.replace(/\s+/g, "").split(";");
    styleArr.map(function(item) {
      if (item) {
        var arr = item.split(":");
        if (arr.length > 1) {
          dom.style[arr[0]] = arr[1];
        }
      }
    });
  } else {
    Object.keys(text).map(function(key) {
      dom.style[key] = text[key];
    });
  }
}
function checkIsIn(key, list) {
  for (var i = 0, len = list.length; i < len; i++) {
    if (key.indexOf(list[i]) > -1) {
      return true;
    }
  }
  return false;
}
function filterStyleFromText(dom) {
  var list = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : ["width", "height", "top", "left", "bottom", "right", "position", "z-index", "padding", "margin", "transform"];
  var _cssText = dom.style.cssText;
  if (!_cssText) {
    return {};
  }
  var styleArr = _cssText.replace(/\s+/g, "").split(";");
  var ret = {};
  var remain = {};
  styleArr.map(function(item) {
    if (item) {
      var arr = item.split(":");
      if (arr.length > 1) {
        if (checkIsIn(arr[0], list)) {
          ret[arr[0]] = arr[1];
        } else {
          remain[arr[0]] = arr[1];
        }
      }
    }
  });
  dom.setAttribute("style", "");
  Object.keys(remain).map(function(key) {
    dom.style[key] = remain[key];
  });
  return ret;
}
function getStyleFromCsstext(dom) {
  var _cssText = dom.style.cssText;
  if (!_cssText) {
    return {};
  }
  var styleArr = _cssText.replace(/\s+/g, "").split(";");
  var ret = {};
  styleArr.map(function(item) {
    if (item) {
      var arr = item.split(":");
      if (arr.length > 1) {
        ret[arr[0]] = arr[1];
      }
    }
  });
  return ret;
}
function stopPropagation(e) {
  if (e) {
    e.stopPropagation();
  }
}
function scrollTop() {
  return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
}
function scrollLeft() {
  return window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft || 0;
}
function checkTouchSupport() {
  return "ontouchstart" in window;
}
function getEventPos(e) {
  var zoom = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 1;
  if (e.touches && e.touches.length > 0) {
    e = e.touches[0];
  }
  return {
    x: e.x / zoom,
    y: e.y / zoom,
    clientX: e.clientX / zoom,
    clientY: e.clientY / zoom,
    offsetX: e.offsetX / zoom,
    offsetY: e.offsetY / zoom,
    pageX: e.pageX / zoom,
    pageY: e.pageY / zoom
  };
}
function requestAnimationFrame(callback) {
  var _fun = window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame || window.oRequestAnimationFrame || window.msRequestAnimationFrame;
  if (_fun) {
    return _fun(callback);
  }
}
function getHostFromUrl(url) {
  if (typeOf(url) !== "String") {
    return "";
  }
  var results = url.split("/");
  var domain = "";
  if (results.length > 3 && results[2]) {
    domain = results[2];
  }
  return domain;
}
function cancelAnimationFrame(frameId) {
  var _fun = window.cancelAnimationFrame || window.mozCancelAnimationFrame || window.cancelRequestAnimationFrame;
  _fun && _fun(frameId);
}
function isMSE(video) {
  if (!video || !(video instanceof HTMLMediaElement)) {
    return false;
  }
  return /^blob/.test(video.currentSrc) || /^blob/.test(video.src);
}
function isBlob(url) {
  return typeof url === "string" && /^blob/.test(url);
}
function generateSessionId() {
  var did = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : 0;
  var d = new Date().getTime();
  try {
    did = parseInt(did);
  } catch (e) {
    did = 0;
  }
  d += did;
  if (window.performance && typeof window.performance.now === "function") {
    d += parseInt(window.performance.now());
  }
  var uuid = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function(c) {
    var r = (d + Math.random() * 16) % 16 | 0;
    d = Math.floor(d / 16);
    return (c === "x" ? r : r & 3 | 8).toString(16);
  });
  return uuid;
}
function createEvent(eventName) {
  var event2;
  if (typeof window.Event === "function") {
    event2 = new Event(eventName);
  } else {
    event2 = document.createEvent("Event");
    event2.initEvent(eventName, true, true);
  }
  return event2;
}
function adjustTimeByDuration(time, duration, isEnded) {
  if (!duration || !time) {
    return time;
  }
  if (time > duration || isEnded && time < duration) {
    return duration;
  }
  return time;
}
function createPositionBar(className, root) {
  var dom = createDom("xg-bar", "", {
    "data-index": -1
  }, className);
  root.appendChild(dom);
  return dom;
}
function getTransformStyle() {
  var pos = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : {
    x: 0,
    y: 0,
    scale: 1,
    rotate: 0
  };
  return "scale(".concat(pos.scale || 1, ") translate(").concat(pos.x || 0, "%, ").concat(pos.y || 0, "%) rotate(").concat(pos.rotate || 0, "deg)");
}
function convertDeg(val) {
  if (Math.abs(val) <= 1) {
    return val * 360;
  }
  return val % 360;
}
function getIndexByTime(time, segments) {
  var _len = segments.length;
  var _index = -1;
  if (_len < 1) {
    return _index;
  }
  if (time <= segments[0].end || _len < 2) {
    _index = 0;
  } else if (time > segments[_len - 1].end) {
    _index = _len - 1;
  } else {
    for (var i = 1; i < _len; i++) {
      if (time > segments[i - 1].end && time <= segments[i].end) {
        _index = i;
        break;
      }
    }
  }
  return _index;
}
function getOffsetCurrentTime(currentTime, segments) {
  var index = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : -1;
  var _index = -1;
  if (index >= 0 && index < segments.length) {
    _index = index;
  } else {
    _index = getIndexByTime(currentTime, segments);
  }
  if (_index < 0) {
    return -1;
  }
  var _len = segments.length;
  var _segments$_index = segments[_index], start = _segments$_index.start, end = _segments$_index.end, cTime = _segments$_index.cTime, offset = _segments$_index.offset;
  if (currentTime < start) {
    return cTime;
  } else if (currentTime >= start && currentTime <= end) {
    return currentTime - offset;
  } else if (currentTime > end && _index >= _len - 1) {
    return end;
  }
  return -1;
}
function getCurrentTimeByOffset(offsetTime, segments) {
  var _index = -1;
  if (!segments || segments.length < 0) {
    return offsetTime;
  }
  for (var i = 0; i < segments.length; i++) {
    if (offsetTime <= segments[i].duration) {
      _index = i;
      break;
    }
  }
  if (_index !== -1) {
    var start = segments[_index].start;
    if (_index - 1 < 0) {
      return start + offsetTime;
    } else {
      return start + (offsetTime - segments[_index - 1].duration);
    }
  }
  return offsetTime;
}
function getLang() {
  var lang = (document.documentElement.getAttribute("lang") || navigator.language || "zh-cn").toLocaleLowerCase();
  if (lang === "zh-cn") {
    lang = "zh";
  }
  return lang;
}
function checkIsCurrentVideo(element, playerId, key) {
  if (!element) {
    return;
  }
  var pid = element.getAttribute(key);
  if (pid && pid === playerId && (element.tagName === "VIDEO" || element.tagName === "AUDIO")) {
    return true;
  }
  return false;
}
export { Hex2RGBA, addClass, adjustTimeByDuration, cancelAnimationFrame, checkIsCurrentVideo, checkIsFunction, checkIsObject, checkTouchSupport, classNames, clearAllTimers, clearInterval, clearTimeout, convertDeg, copyDom, createDom, createDomFromHtml, createEvent, createImgBtn, createPositionBar, deepCopy, deepMerge, event, filterStyleFromText, findDom, format, generateSessionId, getBgImage, getCss, getCurrentTimeByOffset, getEventPos, getFullScreenEl, getHostFromUrl, getIndexByTime, getLang, getOffsetCurrentTime, getStyleFromCsstext, getTransformStyle, hasClass, hide, isBlob, isMSE, isNotNull, isUndefined, padStart, removeClass, requestAnimationFrame, scrollLeft, scrollTop, setInterval, setStyleFromCsstext, setTimeout, show, stopPropagation, toggleClass, typeOf };
